/*
 * $Id: FormHandling.js,v 1.24.2.1 2005/06/27 11:42:29 shanmugampl Exp $
 */

/**
 * IMPORTANT !!!
 * 
 * Always declare a var and then use them. If we are going to assign some
 * variables, then ie throws some exception when not declared. In
 * validateElement method, error was thrown in getAttribute("errormsg") line.
 * The fix was in defining the message outside.
 */


/**
 * Executes the method specified through the functionName argument. The method
 * specified by the functionName should be a part of the Object icorresponding
 * to the formObj. The object should be names as "<FormName/ViewName>Obj". If
 * present the method will be executed.
 *
 * @param	functionName	the function to be executed.
 * @param	formObj				the form for which the function is to be executed.
 */
function executeFunction(functionName, formObj) 
{
  if(functionName == null || formObj == null) {
    return;
  }
  var formObjName = formObj.name + "Obj";
  var result = true;
  if(parent[formObjName] && (parent[formObjName][functionName]))
  {
      result = parent[formObjName][functionName](formObj);
  }
  else
  {
     result = validateForm(formObj);
  }
  if(result)
  {
    return handleStateForForm(formObj);
  }
  else
  {
    return false;
  }
}



/**
 * Checks whether a form object is defined for the form specified by the
 * formName attribute. If not present, creates one and adds it to the parent
 * object. Also defines some default methods.
 *
 * @param	formName	the name of the form for which the object is to be created.
 */
function createFormMethodContainer(formName){
  var methodContainer = null;
  if(parent[formName + "Obj"] != null){
    methodContainer = parent[formName + "Obj"];
  }
  else {
    methodContainer = new Object();
    parent[formName + "Obj"] = methodContainer;
  }

  // aDD the default validateForm method if not present. This method will be
  // called during the form submission.
  if(methodContainer[ "ValidateForm"] == null){
    methodContainer["ValidateForm"] = validateForm;
  }
}

/**
 *
 **/
function validateAndSubmitForm(frm)
{
  return executeFunction("validateForm",frm);   
}

/**
 * A default validation method which iterates through all the elements of the
 * form, looks whether any validation method is specified for the element and
 * if present executes it. Also this takes care of updating the state cookie
 * also.
 *
 * @param	formObject	the form object which is to be validated.
 */
function validateForm(formObject)
{
  var size = formObject.elements.length;
	var isFailure = false;
	// In case the alerttype is CompleteRight/Bottom, the do not return in case
	// of a element validation failure. Execute all the validations and finalyy
	// retuun false;
  for(var count = 0; count < size; count++){
    var element = formObject.elements[count];
    var validateMethod = element.getAttribute("validatemethod");
    if(validateMethod != null && validateMethod != "" && !element.disabled){
			if(!executeElementCall(validateMethod,element)){
				isFailure = true;
				var alertType = formObject.getAttribute("alerttype");
				if(alertType == null || alertType.indexOf("Complete") < 0){
					return false;
				}
			}
    }
  }
	if(isFailure){
		return false;
	}
  return true;
}

/**
 * Executes the given method by passing the formElement as the argument.
 */
function executeElementCall(methodName, formElement){
  var formObj = formElement.form;
  var formName = formObj.getAttribute("viewName");
  if(parent[formObj.name + "Obj"] != null && parent[formObj.name + "Obj"][methodName] != null){
    return parent[formObj.name + "Obj"][methodName](formElement);
  }
  else
  {
    return validateElement(methodName,formElement);
  }
}

// --------------------------- Default Functions --------------------------- //

/**
 * Disables the EditMode for a form and enables the ReadMode
 */
function disableEditMode(value, formElement){
  var formObj = formElement.form;
  var formName = formObj.getAttribute("viewName");
  document.getElementById(formName + "ReadMode").className = 'show';
  document.getElementById(formName + "EditMode").className = 'hide';
}

/************ File Used for Validation ***********/
/*
  trimAll(str)
  Input Parameter :str
  Purpose         : remove all white spaces in front and back of string
  Return          : str without white spaces    
  isEmailId(str)
  Input   :  str
  purpose :  To validate for email
  output  :  valid email true/false
  isEmpty(str)
  input    : str
  purpose  : To check if empty
  output   : if empty true or false
  isIpAddress(str)
  input   : str
  purpose : To check the ip Address
  output  : returns true or false
  isInteger(str)
  input   : str
  purpose : check if number is integer
  output  : returns true or false
  isPositiveInteger(str)
  input   : str
  purpose : check if number is positive integer
  output  : returns true or false
  isDate(str)
  input  : str
  purpose : To check if given str is in date format
  output  : true or false
  isCharacter(str)
  input : str
  purpose : To check if given str contains only characters
  output : true or false
  isTrueFalse(str)
  input : str
  purpose : To check if given str contains only true or false 
  output : true or false
  isInArray(str,arr)
  input : str
  purpose : To check if given string str is contained in the given array arr.
  output : true or false
*/



/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
function trimAll(str)
{
  /*************************************************************
	Input Parameter :str
	Purpose         : remove all white spaces in front and back of string
	Return          : str without white spaces    
  ***************************************************************/
  if(str == null)
  {
    return "";
  }  

  //check for all spaces
  var objRegExp =/^(\s*)$/;
  if (objRegExp.test(str))
  {
    str = str.replace(objRegExp,''); 
    if (str.length == 0)
      return str; 
  } 

  // check for leading and trailling spaces
  objRegExp = /^(\s*)([\W\w]*)(\b\s*$)/;
  if(objRegExp.test(str))
  {
    str = str.replace(objRegExp, '$2');
  }
  return str;
}

/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
function isEmailId(str)
{
  /*****************************************************************
	Input   :  str
	purpose :  To validate for email
	output  :  valid email true/false
  *******************************************************************/
  str = trimAll(str);
  var objRegExp  = /^[a-z0-9]([a-z0-9_\-\.]*)@([a-z0-9_\-\.]*)(\.[a-z]{2,3}(\.[a-z]{2}){0,2})$/i;
  return objRegExp.test(str); 
}

/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
function isMultipleEmailId(str)
{
  /*****************************************************************
	Input   :  str
	purpose :  To validate for multiple email ids
	output  :  valid email true/false
  *******************************************************************/
  var emailIds = str.split(',');
  for(var i = 0; i < emailIds.length; i++)
  {
     if(!isEmailId(emailIds[i]))
     {
        return false;
     }
  }
  return true;
}


/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
function isNotEmpty(str)
{
  /*************************************************************
	input    : str
	purpose  : To check if empty
	output   : if empty true or false
  ***************************************************************/  
  var temp = trimAll(str);
  if (temp.length > 0 )
    return true;
  return false;
}


/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
function isIpAddress(str)
{
  /***************************************************************
	input   : str
	purpose : To check the ip Address
	output  : returns true or false
  *****************************************************************/
	
	
  // we removed the usage of regular expression
  // as some of the browsers ( NN in Sun OS )
  // crashes when doing this validation.
  // now we use simple check for doing the same.

  var ipAddress = str.split(".");
  if(ipAddress.length != 4)
  {
    return false;
  }
  for(i=0;i<ipAddress.length;i++)
  {
    if(isPositiveInteger(ipAddress[i]))
    {
      var temp = parseInt(ipAddress[i],10);
      if(temp > 255)
      {
        return false;
      }
    }
    else
    {
      return false;
    }

  }
  return true;
}


/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
function isInteger(str)
{
  /****************************************************************
	input   : str
	purpose : check if number is integer
	output  : returns true or false
  *****************************************************************/
  var objRegExp = /(^-?\d\d*$)/;
  return objRegExp.test(str);
}

function isPositiveInteger(str)
{
  /****************************************************************
	input   : str
	purpose : check if number is positive integer
	output  : returns true or false
  *****************************************************************/
  var temp  = parseInt(str,10);
  if ( isNaN(temp) || temp.toString().length != str.length) {
    return false;
  }
  var objRegExp = /(^\d\d*$)/;
  return objRegExp.test(temp);
}


/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
function isDate(str)
{
  /***************************************************************
	input  : str
	purpose : To check if given str is in date format
	output  : true or false
  ***************************************************************/
  var objRegExp = /^\d{1,2}(\-|\/|\.)\d{1,2}\1\d{4}$/
                                                     return objRegExp.test(str); 
}

/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
function isCharacter(str)
{
  /***************************************************************
	input : str
	purpose : To check if given str contains only characters
	output : true or false
  ******************************************************************/
  var objRegExp = /^\w$/;
  return objRegExp.test(str);
}

/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
function isTrueFalse(str)
{
  /***************************************************************
	input : str
	purpose : To check if given str contains only true or false 
	output : true or false
  ******************************************************************/
  if((str!="true")&&(str!="false"))
  {
    return false;
  }	
  else
  {
    return true;
  }
}

/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
function isInArray(str,arr)
{
  /***************************************************************
	input : str
	purpose : To check if given string str is contained in the given array arr.
	output : true or false
  ******************************************************************/
  var len =arr.length;
  for(var i =0;i<len;i++)
  {
    if (str==arr[i])
    {
      return true;
    }
  }
  return false;
}


/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

/* Checks whether the value is within a particular range. The range is got from the maxvalue and minvalue
 attributes of the form element
*/
function isWithinRange(value,formElement)
{
	if(!isInteger(value))
	{
		return false;
	}

	value = parseInt(value);

	if(formElement.getAttribute("maxvalue") != null)
	{
		maxValue = parseInt(formElement.getAttribute("maxvalue"));
		if(value > maxValue)
			return false;
	}

	if(formElement.getAttribute("minvalue") != null)
	{
		minValue = parseInt(formElement.getAttribute("minvalue"));
		if(value < minValue)
			return false;
	}

	return true;
}

/**
 * Validates the value entered in the formElement by executing method specified
 * by the methodName argument. If validation fails, then if any errormsg is
 * specified, then an alert is generated for the same else returned. While
 * displaying the alert message, the ${COLUMNALIAS} is replaced by the
 * displayname attribute of the formElement. 
 *
 * The method also supports multiple ways of displaying validation failures
 * depending on the form's alerttype. They are
 * 1. Noraml browser's alert -- Default
 * 2. Custom Alert -- Custom
 * 3. Messages to the right of the field -- CompleteRight
 * 4. Messages to the bottom of the field -- CompleteBottom
 *
 * Of these the CompleteRight and CompleteBottom will show all the error
 * messages at one go whereas the rest will show the results one by one.
 */
function validateElement(methodName,formElement)
{
	if(parent[methodName] == null)
	{
		alert("Wrong configuration.The validation method : " + methodName + " is not loaded");
		return false;
	}
	// the result of the method is fetched separately as a variable because it
	// can either be a boolean or a string. If the result is true, it will be
	// returned.
	var message = null;
	var result = parent[methodName](formElement.value, formElement);
	if(result == true)
	{
		var alertType = formElement.form.getAttribute("alerttype");
		// In case of CompleteRight or Bottom, remove the innerHTML / change the className
		if(alertType != null && alertType.indexOf("Complete")>=0){
			document.getElementById(formElement.id+"_DIV").className = "hide";
		}
		return true;
	}
	else if(result == false)
	{
		// If false is returned, the the specified errormsg should be used for
		// displaying the errors.
		message = formElement.getAttribute("errormsg");
	}
	else 
	{
		// Then what stupid, the returned value is the message itself. Use it.
		message = result;
	}
	if(message != null) 
	{
		if(formElement.getAttribute("displayname") != null)
		{
			message = message.replace("${COLUMNALIAS}", formElement.getAttribute("displayname"));
		}
		var alertType = formElement.form.getAttribute("alerttype");
		if(alertType != null && alertType == 'Custom')
		{
			showCustomMessage(message, formElement);
		}
		else if(alertType != null && alertType.indexOf("Complete")>=0){
			document.getElementById(formElement.id+"_DIV").innerHTML = message;
			document.getElementById(formElement.id+"_DIV").className = "errorMsg show";
		}
		else 
		{
			// Stop! Stop ! Stop. Dont think this as a debugging alert. This is a
			// functionality alert. So do not remove
			alert(message);
		}
	}
	return false;
}

/**
 * Returns the html code for a custom alert.
 */
function getCustomAlert(cssName){
	if(cssName == null){
		cssName = 'customAlert';
	}	
	var code = "<Table class='"+cssName+"' cellspacing=0><tr><td class='caTopLeft'></td><td class='caTopCenter' colspan='2'></td><td class='caTopRight'></td></tr>";
	code = code.concat("<tr><td class='caMiddleLeft'></td><td class='caMessage'>$MESSAGE</td><td class='caClose'><button class='caCloseButton' onClick='hideCustomMessage()'></td><td class='caMiddleRight'></td></tr>");
	code = code.concat("<tr><td class='caBottomLeft'></td><td class='caBottomCenter' colspan='2'></td><td class='caBottomRight'></td></tr>");
	code = code.concat("</table>");
	return code;
}

function showCustomMessage(message, curId, cssName){
   var obj = document.getElementById("customAlertMessage");
   var ifrmObj = document.getElementById("customAlertFrame");
   if(obj == null){
       var newObj = document.createElement("DIV");
       if(browser_ie){
           newObj.id="customAlertMessage";
           newObj.style="position: absolute; cursor: default;"
           newObj.className = "hide";
       }
       else if (browser_nn4 || browser_nn6){
           newObj.setAttribute("id","customAlertMessage");
           newObj.setAttribute("style","position: absolute; cursor: default;");
           newObj.setAttribute("className","hide");
       }
       document.body.appendChild(newObj);
       obj = newObj;
   }
   var code = getCustomAlert(cssName);
   code = code.replace("$MESSAGE", message);
   ifrmObj.style.display = 'block';
   obj.innerHTML = code;
   obj.style.left = findPosX(curId);
   obj.style.top = findPosY(curId) + curId.offsetHeight;
   obj.style.width = curId.parentNode.offsetWidth;
   obj.style.zIndex = 100;
   ifrmObj.style.left = findPosX(curId);
   ifrmObj.style.top = findPosY(curId) + curId.offsetHeight;
   ifrmObj.style.width = curId.parentNode.offsetWidth;
   ifrmObj.style.height = 40;
   ifrmObj.style.zIndex = obj.style.zIndex - 1;
   obj.className = 'show';
} 

function hideCustomMessage(){
   document.getElementById("customAlertFrame").style.display = "none";
   document.getElementById("customAlertMessage").className = "hide";
}

function isNumeric(str, formElement){
    var objRegExp = /^[0-9]+$/;
    if(objRegExp.test(str)){
        return true;
    }
    return false;
}

function isAlphaNumeric(str, formElement){
    var objRegExp = /^[a-zA-Z0-9]+$/;
    if(objRegExp.test(str)){
        return true;
    }
    return false;
}

function isMinute(str, formElement){
	str = trim(str);
	if(str==""){
		return false;
	}
	var regEx = /^[0-9]+$/;
	if(regEx.test(str)){
		if(str <= 59){
			return true;
		}
	}
	return false;
}

function isHour(str, formElement){
	str = trim(str);
	if(str==""){
		return false;
	}
	var regEx = /^[0-9]+$/;
	if(regEx.test(str)){
		if(str <= 23){
			return true;
		}
	}
	return false;
}

function isSearchDataNumeric(str, formElement){
	trimAll(str);
	if(str != null && str != ''){
		return isNumeric(str, formElement);
	}
	return true;
}
