/*
 * $Id: MenuAPI.js,v 1.45.2.1 2005/05/26 10:56:23 shanmugampl Exp $
 *
 * Copyright (c) 2002 Adventnet, Inc. All Rights Reserved.
 * Please read the associated COPYRIGHTS file for more details.
 *
 * ADVENTNET, INC. MAKES NO REPRESENTATIONS OR WARRANTIES  ABOUT THE SUITABILITY
 * OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. ADVENTNET, INC. SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS
 * SOFTWARE OR ITS DERIVATIVES.
 *
 */

/*
 * This is the function that will be invoked by default for all the actions 
 * that are taken care by the menus.         
 */

function invokeAction(menuItemId, id, tableAlias, reqParams)
{
	var uniqueId = null;
	if(id != null)
	{
		uniqueId = getUniqueId(id);
		tableAlias = getTableNameForMenuItem(menuItemId, uniqueId, tableAlias);
		if(false == isAtleastOneChecked(menuItemId, uniqueId))
		{   
			return;
		}
	}
	if(false == confirmAction(menuItemId))
	{
		return;
	}
	var url = getActionURL(menuItemId, uniqueId, tableAlias, uniqueId);
	if(reqParams != null){
		url = url.concat("&"+reqParams);
	}
	openActionURL(menuItemId, url, uniqueId);
}

function isAtleastOneChecked(menuItemId, id)
{
	/*
	 * If no table alias is configured for this menu item,then it is assumed that
	 * this does not depend on any selection in the browser. So, allow it to 
	 * proceed with the action.
	 */
	var uniqueId = getUniqueId(id);
	if(menuItemId.TABLEALIAS == null)
	{
		return true;
	}
	var atleastOneChecked = null;
	var frm = getForm(uniqueId);
	if(frm != null)
	{
		/*
		 * This attribute identifies whether the check has to be made for the current
		 * action. Here a crude assumption is made that the name "rowSelection" is 
		 * exclusively reserved for the table based views. 
		 */
		var isRowSelectionPresent = false;
		// the isToggleAllPresent is specifically used to identify a table
		// component. This along with the count attribute will be used to determine
		// whether a table has rows or not.
		var isToggleAllPresent = false;
		atleastOneChecked = false;
		var len = frm.elements.length;
		var count = 0;
		for (var i = 0; i < len; i++)
		{
			var e = frm.elements[i];
			if(e.name == "toggleAll"){
				isToggleAllPresent = true;
			}
			if (e.name == "rowSelection")
			{
				count++;
				isRowSelectionPresent = true;
				if (e.checked)
				{
					atleastOneChecked = true;
				}
			}
		}

		if(!atleastOneChecked && !isRowSelectionPresent)
		{
			atleastOneChecked = true;
		}

	}

	/*
	 * This check's whether any data is present in the table or not. If present
	 * then the count will be greater than 0. If this check fails then the
	 * execution will stop with an alert.
	 */
	if(count == 0 && isToggleAllPresent){
		alert("No Rows present in the table");
		return false;
	}
	if(atleastOneChecked != null)
	{
		if (atleastOneChecked == false)
		{
			alert("Select atleast one row to " + menuItemId.DISPLAYNAME);
			return false;
		}
	}
	return true;
}

function getActionURL(menuItemId, id, tableAlias)
{
	var uniqueId = getUniqueId(id);
    var url = menuItemId.ACTIONLINK;
    url = getURLSuffixed(url);
    if(uniqueId != null)
    {
        url += "ACTION_SOURCE=" + uniqueId + "&";
    }
    if((tableAlias != null) && (uniqueId != null))
    {
        var frm = getForm(uniqueId);
        if(uniqueId != null && uniqueId != "null" && uniqueId != "")
        {
            if(frm != null)
            {
                var len = frm.elements.length;
				var rowIndices = new Array(0);
                for (var i = 0; i < len; i++)
                {
                    var e = frm.elements[i];
                    if (e.name == "rowSelection")
                    {
                        if (e.checked)
                        {
                            //var rowIndex = e.value;
							rowIndices.push(e.value);
                            /*var rowDatum = rowData[rowIndex];
                            for (var j=0; j<pkCols.length; j++)
                            {
                                url += pkCols[j] + "=" + rowDatum[pkCols[j]] + "&";
                            }*/
                        }
                    }
                }
				url += getParams(rowIndices, uniqueId, tableAlias);
            }
            else
            {
				url += getParams(null, uniqueId, tableAlias);
            }
        }
    }
    return url;
}

function getParams(rowIndices, id, tableAlias)
{
	var uniqueId = getUniqueId(id);
	var params = "";
	var pkCols = getPKColumnNames(uniqueId, tableAlias);
	var rowData = getDataValueArray(uniqueId, tableAlias);
	if(rowIndices != null && rowIndices.length > 0)
	{
		for(var i=0; i<rowIndices.length; i++)
		{
			var rowDatum = rowData[rowIndices[i]];
        	for (var j=0; j<pkCols.length; j++)
			{
				params += pkCols[j] + "=" + rowDatum[pkCols[j]] + "&";
			}
		}
	}
	else
	{
		pkCols = eval(uniqueId + "_" + tableAlias + "_PKColumnName");
		for(var i=0; i<rowData.length; i++)
		{
			var rowDatum = rowData[i];
			for(var j=0; j<pkCols.length; j++)
			{
				params += "&" + pkCols[j]+ "=" + rowDatum[pkCols[j]];
			}
		}
	}
	return params;
}

/*
* Confirms the action performed by the user in a dialog box. Returns the value 
* appropriately to the caller based on the users response of clicking "Yes" or 
* "No" buton.
*/
function confirmAction(menuItemId)
{
    var cnfAction = menuItemId.CONFIRMSTRING;
    var cnf = null;
    if(cnfAction)
    {
        cnf = confirm(cnfAction);
    }
    return cnf;
}

function invokeActionFromView(menuItemId, id, tableAlias, index)
{
    var uniqueId = getUniqueId(id);
    var url = getInvokeActionFromViewURL(menuItemId, uniqueId, tableAlias, index);
    openActionURL(menuItemId, url, uniqueId);
}

function getInvokeActionFromViewURL(menuItemId, id, tableAlias, index)
{
	var uniqueId = getUniqueId(id);
    if(checkForNull(tableAlias))
    {
        tableAlias = menuItemId.TABLEALIAS;
    }
   return getInvokeActionFromView(menuItemId.ACTIONLINK,uniqueId,tableAlias,index);
}

function evalViewFun(funName,funString)
{
  if(parent[funName])
  {
    return;
  }
  parent[funName] = new Function("index","viewName",funString);
}

function getInvokeActionFromView(actUrl,id,tableAlias,index)
{
	var uniqueId = getUniqueId(id);
    tableAlias = getTableName(uniqueId, tableAlias);
    var rowDatum = getDataValue(uniqueId, tableAlias, index);
    var pkCols = getPKColumnNames(uniqueId, tableAlias);
    actUrl+=(actUrl.indexOf("?") <0 ) ? "?":"&";
    for (var j=0; j<pkCols.length; j++)
    {
        actUrl += "&" + pkCols[j] + "=" + rowDatum[pkCols[j]];
    }   
    actUrl += "&ACTION_SOURCE=" + uniqueId + "&";
    return actUrl;
}

function openActionURL(menuItemId, url, id)
{
	var uniqueId = getUniqueId(id);
	var winPrams = menuItemId.WINPARAMS;
	var target = menuItemId.TARGET;
	var winParams = menuItemId.WINPARAMS;
	var target = menuItemId.TARGET;
	var refreshLevel = menuItemId.REFRESHLEVEL;
	if((target == null) || (target == '_self'))
        {
           processURLBasedOnLevel(url, uniqueId, refreshLevel);
	}
        else
        {
	   openURL(url, target, winParams);
        }
}

function setMenuItemProperties(menuItem, properties)
{
    menuItem.MENUITEMID = properties[0];
    menuItem.DISPLAYNAME = properties[1];
    menuItem.IMAGE = properties[2];
    menuItem.CONFIRMSTRING = properties[3];
    menuItem.ACTIONLINK = properties[4];
    menuItem.TABLEALIAS = properties[5];
    menuItem.TARGET = properties[6];
    menuItem.WINPARAMS = properties[7];
    menuItem.JSMETHODNAME = properties[8];
    menuItem.REFRESHLEVEL = properties[9]
}

function setMenuDescription(menuItem, desc)
{
    menuItem.DESCRIPTION = desc;
}

function getForm(id)
{   
	var uniqueId = getUniqueId(id);
    var frmName = uniqueId + "_FRM";
    for(var i=0; i<document.forms.length; i++)
    {
        if(document.forms[i].name == frmName)
        {
            return document.forms[i];
        }
    }
    return null;
}

function getTableNameForMenuItem(menuItemId, id, tableAlias)
{
	var uniqueId = getUniqueId(id);
    if(checkForNull(tableAlias))
    {
        tableAlias = menuItemId.TABLEALIAS;
    }
    return getTableName(uniqueId,tableAlias);
}

function getTableName(id,tableAlias)
{
    if(tableAlias != null && !checkForNull(uniqueId))
    {
	var uniqueId = getUniqueId(id);
	var viewName = stateData[uniqueId]["_VN"];
       /*
        * This check is necessary because when acting upon the property sheets
        * there may not be table aliases at all.
        */
        if(! eval("window."+viewName+"_TableAliases"))
        {
            return tableAlias;
        }
        
        var tableAliases = eval(viewName+"_TableAliases");
        var aliasPresent = false;
        for(var i=0; i<tableAliases.length; i++)
        {
            if(tableAliases[i] == tableAlias)
            {
                aliasPresent = true;
                break;
            }
        }
        if(! aliasPresent)
        {
            var tableNames = eval(viewName+"_TableNames");
            for(var i=0; i<tableNames.length; i++)
            {
                if(tableNames[i] == tableAlias)
                {
                    tableAlias = tableAliases[i];
                    break;
                }
            }
        }
    }
    return tableAlias;
}

function getURLSuffixed(url)
{
    if(url.lastIndexOf('?') == -1)
    {
        url += "?";
    }
    else if(url.indexOf('?') > 0 && url.charAt(url.length-1) != '&')
    {
        url += "&";
    }
    return url;
}

function checkForNull(toBeChecked)
{
    if(toBeChecked == null || toBeChecked == "null" || toBeChecked == "")
    {
        return true;
    }
    else
    {
        return false;
    }
}

function testMenuItemObj(menuItemObj)
{
    alert("ID --> " + menuItemObj.MENUITEMID);
    alert("DisplayName --> " + menuItemObj.DISPLAYNAME);
    alert("Table Name --> " + menuItemObj.TABLEALIAS)
}

function getSelectedRowIndices(id)
{
	var uniqueId = getUniqueId(id);
    var selectedIndices = new Array();
    var frm = getForm(uniqueId);
    var len = frm.elements.length;
    var arraySize = 0;
    for (var i = 0; i < len; i++)
    {
        var e = frm.elements[i];
        if (e.name == "rowSelection")
        {
            if (e.checked)
            {
                selectedIndices[arraySize++] = e.value;
            }
        }
    }
    return selectedIndices;
}

